@@ -1,7 +1,7 @@
 #' @name Q.test
 #' @rdname Q.test
-#' @title A function to compute Q test for spatial qualitative data using
-#'        asymptotic distribution
+#' @title A function to compute Q test for spatial qualitative data
+#' @description A function to compute Q test for spatial qualitative data.
 #' @usage Q.test(formula = NULL, data = NULL, na.action,
 #' fx = NULL, coor = NULL, m = 3, r = 1, distr = "asymptotic",
 #' control = list())
@@ -11,13 +11,13 @@
 #'             variable(s) to be tested.
 #' @param fx  a factor  or a matrix of factors in columns
 #' @param na.action action with NA values
-#' @param m length of m-surrounding (default = 3).
-#' @param r maximum overlapping between any two m-surroundings (default = 1).
 #' @param distr character. Distribution type "asymptotic" (default) or "mc".
+#' @param m length of m-surrounding (default = 3).
+#' @param r only for asimtotic distribution. Maximum overlapping between any two m-surroundings (default = 1).
 #' @param coor (optional) a 2xN vector with spatial coordinates.
 #'             Used when *data* is not a spatial object
 #' @param control Optional argument. See Control Argument section.
-#' @description A function to compute Q test for spatial qualitative data
+#'
 #' @return An list of two object of the class \code{htest}. Each element of the list return the:
 #'   \tabular{ll}{
 #'     \code{data.name} \tab a character string giving the names of the data.\cr
@@ -39,38 +39,137 @@
 #' \item{dtmaxabs}{Delete degenerate surrounding based on the absolute distance between observations.}
 #' \item{dtmaxpc}{A value between 0 and 1. Delete degenerate surrounding based on the distance. Delete m-surrounding when the maximum distance between observation
 #' is upper than k percentage of maximum distance between anywhere observation.}
-#' \item{dtmaxknn}{A integer value 'k'. Delete degenerate surrounding based on the near neighborhood criteria. Delete m-surrounding
-#' is a element of the m-surrounding is not include in the set of k near neighborhood of the first element}
+#' \item{dtmaxknn}{A integer value 'k'. Delete degenerate surrounding based on the near neighbourhood criteria. Delete m-surrounding
+#' is a element of the m-surrounding is not include in the set of k near neighbourhood of the first element}
 #' \item{nsim}{number of simulations for get the Monte Carlo distribution.
 #'  Default = 999}
 #' \item{seedinit}{seed to select the initial element to star
 #' the algorithm to get compute the m-surroundings or to start the simulation}
 #' }
 #'
+#' @details
+#' The Q(m) statistic was introduced by Ruiz et al. (2010) as a tool to explore geographical
+#' co-location/co-occurrence of qualitative data. Consider a spatial variable X which is the
+#' result of a qualitative process with a set number of categorical outcomes \eqn{a_j} (j=1,...,k).
+#' The spatial variable is observed at a set of fixed locations indexed by their coordinates
+#' \eqn{s_i} (i=1,..., N), so that at each location si where an event is observed,
+#' \eqn{X_i} takes one of the possible values \eqn{a_j}.\cr
+#'
+#' Since the observations are georeferenced, a spatial embedding protocol can be devised
+#' to assess the spatial property of co-location. Let us define, for an observation at
+#' a specified location, say \eqn{s_0}, a surrounding of size m, called an m-surrounding.\cr
+#' The m-surrounding is the set of m-1 nearest neighbours from the perspective
+#' of location \eqn{s_0}. In the case of distance ties, a secondary criterion can be
+#' invoked based on direction.\cr
+#' Once that an embedding protocol is adopted and the elements of the m-surrounding
+#' for location \eqn{s_0} have been determined, a string can be obtained that collects
+#' the elements of the local neighborhood (the m-1 nearest neighbors) of the observation
+#' at \eqn{s_0}. The m-surrounding can then be represented in the following way:
+#'
+#' \deqn{X_m(s_0)=(X_{s_0},X_{s_1},...X_{s_{m-1}})}
+#' Since each observation Xs takes one of k possible values, and there are m observations in
+#' the m-surrounding, there are exactly k possible unique ways in which those values can
+#' co-locate. This is the number of permutations with replacement.\cr
+#' For instance, if k=2
+#' (e.g. the possible outcomes are a1=0 and a2=1) and m=3, the following eight unique
+#' patterns of co-location are possible (the number of symbols is \eqn{n_{\sigma}}=8): {0,0,0}, {1,0,0},
+#' {0,1,0}, {0,0,1}, {1,1,0}, {1,0,1}, {0,1,1}, and {1,1,1}. Each unique co-locationtype
+#' can be denoted in a convenient way by means of a symbol \eqn{\sigma_i} \eqn{(i=1, 2,...,k^m)}. It follows
+#' that each site can be uniquely associated with a specific symbol, in a process termed
+#' symbolization. In this way, we say that a location s is of type \eqn{\sigma_i} if and only if \eqn{X_m(s)=\sigma_i}.\cr
+#' Equivalent symbols (see Páez, et al. 2012) can be obtained by counting the number of
+#' occurrences of each category within an m-surrounding. This surrenders some
+#' topological information (ordering within the m-surrounding is lost) in favor of a more
+#' compact set of symbols, since the number of combinations with replacement.\cr
+#' \cr
+#' \strong{Definition of Q(m) statistic}\cr
+#' \cr
+#' Let \eqn{\{X_s\}_{s \in R}} be a discrete spatial process and m be a fixed embedding
+#' dimension. The statistic Q testing the null hypothesis:\cr
+#' \cr
+#' \eqn{H_0:\{X_s\}_{s \in R}} is spatially independent, against any other alternative.\cr
+#' \cr
+#' For a fixed \eqn{m \geq 2}, the relative frequency of symbols can be used to define the symbolic
+#' entropy of the spatial process as the Shanon entropy of the distinct symbols:\cr
+#' \deqn{h(m) = - \sum_j p_{\sigma_j}ln(p_{\sigma_j})}
+#' where
+#' \deqn{p_{\sigma_j}={ n_{\sigma_j} \over R}}
+#' with \eqn{n_{\sigma_j}} is simply the
+#' number of times that the symbol \eqn{\sigma_j} is observed and R the number of
+#' symbolized locations.
+#' The entropy function is bounded between \eqn{0 < h (m) \leq \eta}.\cr
+#'
+#' The Q statistic is essentially a likelihood ratio test between the symbolic entropy
+#' of the observed pattern and the entropy of the system under the null hypothesis
+#' of a random spatial sequence:
+#'  \deqn{Q(m)=2R(\eta-h(m))}
+#' with \eqn{\eta = ln(k^m)}. The statistic is asymptotically \eqn{\chi^2} distributed
+#' with degrees of freedom equal to the number of symbols minus one.\cr
+#'
+#' @section Standard-Permutation vs Equivalent-Combination Symbols:
+#' The symbolization protocol proposed by Ruiz et al. (2010) - call these
+#' Standard-Permutation Symbols — contains a large amount of topological information
+#' regarding the units of analysis, including proximity and direction.
+#' In this sense, the protocol is fairly general. On the other hand, it is easy to see
+#' that the combinatorial possibilities can very quickly become unmanageable.
+#'  For a process with k = 3 outcomes and m = 5, the number of symbols becomes
+#'  \eqn{3^5 = 243}; for k = 6 and m = 4 it is  \eqn{6^4 = 1,296}. Depending on the number
+#'  of observations N, the explosion in the number of symbols can very rapidly consume
+#'  degrees of freedom for hypothesis testing, because as a rule of thumb
+#'  it is recommended that the number of symbolized locations be at least five times
+#'  the number of symbols used (e.g., \eqn{R \geq 5k^m}), and R will usually be a fraction of N.\cr
+#'
+#'  As an alternative, we propose a symbolization protocol that sacrifices
+#'  some amount of topological detail for conciseness. The alternative is based
+#'  on the standard scheme; however, instead of retaining proximity and
+#'  direction relationships, it maintains only the total number of occurrences
+#'  of each outcome in an m-surrounding. We call these Equivalent-Combination Symbols.
+#'  Because order in the sequence is not considered in this protocol, instead of a
+#'  permutation with repetition, the number of symbols reflects a combination with
+#'  repetition. \cr
+#'
+#' @section Selection of m-surrounding with Controlled Degree of Overlapping (r):
+#'  To select S locations for the analysis, coordinates are selected such that
+#'  for any two coordinates \eqn{s_i} , \eqn{s_j} the number of overlapping nearest
+#'  neighbours of \eqn{s_i} and \eqn{s_j} are at most r. The set S, which is a subset of all the
+#'  observations N, is defined recursively as follows. First choose a location \eqn{s_0} at random and fix an integer r
+#'  with \eqn{0 \leq r < m}. The integer r is the degree of overlap, the maximum number of observations that contiguous
+#'  m-surroundings are allowed to have in common.\cr
+#'  Let \eqn{ \{s_1^0, s_2^0,...,s_{m-1}^0 \}} be the set of nearest neighbours
+#'  to \eqn{s_0}, where the \eqn{s_i^0} are ordered by distance to \eqn{s_0}, or angle in the case of ties.
+#'  Let us call \eqn{s_1 = s_{m-r-1}^0} and define \eqn{ A_0 = \{s_0,s_0^1,...,s^0_{m-r-2} \}  } . Take the set of
+#'  nearest neighbours to \eqn{s_1}, namely, \eqn{ \{ s_1^1, s_2^1,...,s^1_{m-1} \} } in the
+#'  set of locations \eqn{S \setminus A_0 } and define \eqn{s_2=s^1_{m-r-1} }. Nor for i>1 we define
+#'  \eqn{s_i = s^{i-1}_{m-r-1}} where \eqn{s^{i-1}_{m-r-1}} is in the ser of nearest neighbors to \eqn{s_{i-1}},
+#'   \eqn{ \{ s_1^{i-1},s_2^{i-1},...,s_{m-1}^{i-1} \} } ,of the set \eqn{S \setminus \{ \cup_{j=0}^{i-1} A_j \} }. Continue this process while there are locations to symbolize.\cr
+#' @section Selection of m-surroundings for bootstrap distribution:
+#' The bootstrapped-based testing can provide an advantage since overlapping between
+#' m-surroundings is not a consideration, and the full sample can be used.\cr
+#'
 #' @author
-#'   \tabular{ll}{
-#'   Fernando López  \tab \email{fernando.lopez@@upct.es} \cr
-#'   Román Mínguez  \tab \email{roman.minguez@@uclm.es} \cr
-#'   Antonio Páez \tab \email{paezha@@gmail.com} \cr
-#'   Manuel Ruiz \tab \email{manuel.ruiz@@upct.es} \cr
+#' \tabular{ll}{
+#' Fernando López  \tab \email{fernando.lopez@@upct.es} \cr
+#' Román Mínguez  \tab \email{roman.minguez@@uclm.es} \cr
+#' Antonio Páez \tab \email{paezha@@gmail.com} \cr
+#' Manuel Ruiz \tab \email{manuel.ruiz@@upct.es} \cr
 #'   }
 #' @references
 #'   \itemize{
-#'     \item Ruiz M, López FA, A Páez. (2010). \emph{Testing for spatial association of qualitative
-#'     data using symbolic dynamics}. Journal of Geographical Systems. 12 (3) 281-309
-#'     \item López, FA, and A Páez. (2012). \emph{Distribution-free inference for Q(m) based on permutational bootstrapping: an application
-#'     to the spatial co-location pattern of firms in Madrid} Estadística Española, 177, 135-156.
+#'     \item Ruiz M, López FA, A Páez. (2010). Testing for spatial association of qualitative
+#'     data using symbolic dynamics. \emph{Journal of Geographical Systems}. 12 (3) 281-309
+#'     \item López, FA, and A Páez. (2012). Distribution-free inference for Q(m) based on permutational bootstrapping: an application
+#'     to the spatial co-location pattern of firms in Madrid \emph{Estadística Española}, 177, 135-156.
 #'   }
 #' @export
 #' @examples
 #'
 #' # Case 1: With coordinates
-#' N <- 100
+#' N <- 200
 #' cx <- runif(N)
 #' cy <- runif(N)
 #' coor <- cbind(cx,cy)
 #' p <- c(1/6,3/6,2/6)
-#' rho = 0.5
+#' rho = 0.3
 #' listw <- spdep::nb2listw(spdep::knn2nb(spdep::knearneigh(cbind(cx,cy), k = 4)))
 #' fx <- dgp.spq(list = listw, p = p, rho = rho)
 #' q.test <- Q.test(fx = fx, coor = coor, m = 3, r = 1)
@@ -78,7 +177,7 @@
 #' plot(q.test)
 #' print(q.test)
 #' \donttest{
-#' q.test.mc <- Q.test(fx = fx, coor = coor, m = 3, r = 1, distr = "mc")
+#' q.test.mc <- Q.test(fx = fx, coor = coor, m = 3, distr = "mc", control = list(nsim = 999))
 #' summary(q.test.mc)
 #' plot(q.test.mc)
 #' print(q.test.mc)
@@ -171,6 +270,11 @@ Q.test <- function(formula = NULL, data = NULL, na.action,
   dtmaxknn <- con$dtmaxknn
   cl <- match.call()
    # Lectura Datos.
+  if (!is.null(formula) && !is.null(coor) && !is.null(data)) {
+    data$Lat <- coor[,1]
+    data$Lon <- coor[,2]
+    data <- sf::st_as_sf(data, coords = c("Lat","Lon"))
+  }
   if (!is.null(formula) && !is.null(data)) {
     if (inherits(data, "Spatial")) data <- as(data, "sf")
     mfx <- model.frame(formula, data, na.action = na.action)
@@ -201,6 +305,7 @@ Q.test <- function(formula = NULL, data = NULL, na.action,
       mj <- m[j]
       symbi <- cr_symb(ki, mj)
       if (distr == "asymptotic") {
+
         rcut <- r[!(r > (mj - 1))]
         lms[[j]] <- vector(mode = "list",
                            length = length(rcut))
@@ -229,6 +334,7 @@ Q.test <- function(formula = NULL, data = NULL, na.action,
           #                        control = list(dtmaxabs = dtmaxabs,
           #                                       dtmaxpc = dtmaxpc))
           # } else stop("Value for argument 'typems' not valid.")
+
           qi <- q_symb_A2(xfi, lms[[j]][[h]]$ms, symbi)
           qi$qp_df <- nrow(symbi$p_symb) - 1
           qi$qc_df <- nrow(symbi$c_symb) - 1
