@@ -1,7 +1,8 @@
 #'
-#' @title Compute the scan test.
+#' @title Compute the scan test
 #'
-#' @description This function compute the scan test for Bernoulli and Multinomial categorical spatial process.
+#' @description This function compute the spatial scan test for Bernoulli and
+#' Multinomial categorical spatial process, and detect spatial clusters
 #'
 #' @param data an (optional) data frame or a sf object containing the variable to testing for.
 #' @param formula a symbolic description of the factor (optional).
@@ -15,28 +16,37 @@
 #' of "High" (default), "Both" or "Low".
 #' @param distr distribution of the spatial process: "bernoulli" for two levels or "multinomial" for three or more levels.
 #' @param windows a string to select the type of cluster "circular" (default) of "elliptic".
+#' @param listw only for flexible windows. A neighbours list (an object of the class listw, nb or knn frop spdep) or an adjacency matrix.
 #' @param minsize Minimum number of observations inside of Most Likely Cluster and secondary clusters.
 #' @param control List of additional control arguments.
-#' @usage scan.test(formula = NULL, data = NULL, fx = NULL, coor = NULL,
-#' case = NULL, nv = NULL, nsim = NULL, distr = NULL, windows = "circular",
+#' @usage scan.test(formula = NULL, data = NULL, fx = NULL, coor = NULL, case = NULL,
+#' nv = NULL, nsim = NULL, distr = NULL, windows = "circular", listw = NULL,
 #' alternative = "High", minsize = 1, control = list())
 #' @details
-#'
 #' Two alternative sets of arguments can be included in this function to compute the scan test:
-#'
-#'   \tabular{ll}{
-#'     \code{Option 1} \tab A factor (fx) and coordinates (coor). \cr
-#'     \code{Option 2} \tab A sf object (data) and the formula to specify the factor.
-#'     The function consider the coordinates of the centroids of the elements of th sf object. \cr
+#' \itemize{
+#'     \item {\code{Option 1}: A factor (fx) and coordinates (coor)}.
+#'     \item {\code{Option 2}: A sf object (data) and the formula to specify the factor.
+#'     The function consider the coordinates of the centroids of the elements of th sf object.}
 #'     }
-#'
 #'  The spatial scan statistics are widely used in epidemiology, criminology or ecology.
-#'  Their purpose is to analyze the spatial distribution of points or geographical
+#'  Their purpose is to analyse the spatial distribution of points or geographical
 #'  regions by testing the hypothesis of spatial randomness distribution on the
 #'  basis of different distributions (e.g. Bernoulli, Poisson or Normal distributions).
-#'  The \code{scan.test} function obtain the scan statistic for the Bernoulli and Multinomial distribution.
-#'
-#'  To test independence in a spatial process, under the null, the type of windows is irrelevant but under the alternative the elliptic
+#'  The \code{scan.test} function obtain the scan statistic for two relevant distributions related
+#'  with categorical variables: the Bernoulli and Multinomial distribution.\cr
+#'  The spatial scan statistic is based on the likelihood ratio test statistic and
+#'  is formulated as follows:\cr
+#'  \deqn{\Delta = { \max_{z \in Z,H_A} L(\theta|z) \over \max_{z \in Z,H_0} L(\theta|z)}}
+#'  where Z represents the collection of scanning windows constructed on the study region,
+#'  \eqn{H_A} is an alternative hypothesis, \eqn{H_0} is a null hypothesis, and \eqn{L(\theta|z)}
+#'   is the likelihood function with parameter \eqn{\theta} given window Z\cr.
+#'   The null hypothesis says that there is no spatial clustering on the study region, and
+#'   the alternative hypothesis is that there is a certain area with high (or low) rates of
+#'   outcome variables. The null and alternative hypotheses and the likelihood function may
+#'   be expressed in different ways depending on the probability model under consideration.\cr
+#'  To test independence in a spatial process, under the null, the type of windows is
+#'  irrelevant but under the alternative the elliptic
 #'  windows can to identify with more precision the cluster.
 #'
 #'  For big data sets (N >>) the windows = "elliptic" can be so slowly
@@ -70,7 +80,6 @@
 #'   \tabular{ll}{
 #'     \code{seedinit} \tab Numerical value for the seed (only for boot version). Default value seedinit=123 \cr
 #'       }
-
 #' @author
 #'   \tabular{ll}{
 #'   Fernando López  \tab \email{fernando.lopez@@upct.es} \cr
@@ -90,8 +99,59 @@
 #'      Clustering and co-occurrence of cancer types: A comparison of techniques with
 #'      an application to pediatric cancer in Murcia, Spain.
 #'      \emph{Spatial Analysis in Health Geography}, 69-90.
-#'
+#'      \item Tango T., Takahashi K. (2005). A flexibly shaped spatial scan statistic
+#'      for detecting clusters, \emph{International Journal of Health Geographics} 4:11.
 #'   }
+#' @details
+#' \strong{Bernoulli version}\cr
+#' \cr
+#' When we have dichotomous outcome variables, such as cases and noncases of certain
+#'  diseases, the Bernoulli model is used. The null hypothesis is written as
+#'  \deqn{H_0 : p = q  \ \ for \ \  all  \ \ Z}
+#'  and the alternative hypothesis is \cr
+#'  \deqn{H_A : p \neq q  \ \ for  \ \ some  \ \ Z}
+#'  where p and q are the outcome probabilities (e.g., the probability of being a case)
+#'  inside and outside scanning window Z, respectively. Given window Z, the test statistic is:\cr
+#'
+#' where cz and nz are the numbers of cases and observations (cases and noncases) within z,
+#'  respectively, and C and N are the total numbers of cases and observations in the whole
+#'  study region, respectively.\cr
+#' \deqn{\Delta = }
+#' \strong{Multinomial version of the scan test}\cr
+#' \cr
+#' The multinomial version of the spatial scan statistic is useful to investigate clustering
+#' when a discrete spatial variable can take one and only one of k possible outcomes
+#' that lack intrinsic order information. If the region defined by the moving window
+#' is denoted by Z, the null hypothesis for the statistic can be stated as follows:\cr
+#' \deqn{H_0: p_1 = q_1;p_2 = q_2;...;p_k = q_k}
+#' where \eqn{p_j} is the probability of being of event type j inside the window Z,
+#' and \eqn{q_j} is the probability of being of event type j outside the window.
+#' The alternative hypothesis is that for at least one type event the probability
+#'  of being of that type is different inside and outside of the window.\cr
+#'
+#'  The statistic is built as a likelihood ratio, and takes the following
+#'  form after transformation using the natural logarithm:
+#'\deqn{\Delta = \max_Z \{\sum_j \{ S_j^Z log({ S_j^Z \over S^Z }) +
+#'(S_j-S_j^Z) log({ {S_j-S_j^Z} \over {S-S^Z} })\}\}-\sum_j S_j log({ S_j \over S }) }
+#' where S is the total number of events in the study area and \eqn{S_j} is the total number
+#' of events of type j. The superscript Z denotes the same but for the sub-region
+#' defined by the moving window. \cr
+#' The theoretical distribution of the statistic under the null hypothesis is not known,
+#'  and therefore significance is evaluated numerically by simulating neutral landscapes
+#'  (obtained using a random spatial process) and contrasting the empirically calculated
+#'  statistic against the frequency of values obtained from the neutral landscapes.
+#'  The results of the likelihood ratio serve to identify the most likely cluster,
+#'  which is followed by secondary clusters by the expedient of sorting them according to
+#'   the magnitude of the test. As usual, significance is assigned by the analyst, and the
+#'    cutoff value for significance reflects the confidence of the analyst, or tolerance for error.\cr
+#'  When implementing the statistic, the analyst must decide the shape of the
+#'  window and the maximum number of cases that any given window can cover.
+#'  Currently, analysis can be done using circular or elliptical windows.\cr
+#'  Elliptical windows are more time consuming to evaluate but provide greater
+#'  flexibility to contrast the distribution of events inside and outside the window,
+#'  and are our selected shape in the analyses to follow. Furthermore, it is recommended
+#'  that the maximum number of cases entering any given window does not exceed 50\% of
+#'  all available cases.
 #' @seealso
 #' \code{\link{local.sp.runs.test}}, \code{\link{dgp.spq}}, \code{\link{Q.test}},
 #' @export
@@ -109,14 +169,14 @@
 #' print(scan)
 #' summary(scan)
 #' plot(scan, sf = provinces_spain)
-#'
+#' \donttest{
 #' ## With maximum number of neighborhood
 #' scan <- scan.test(formula = formula, data = provinces_spain, case = "woman",
 #' nsim = 99, distr = "bernoulli")
 #' print(scan)
 #' plot(scan, sf = provinces_spain)
 #'
-#' \donttest{
+#'
 #' ## With elliptic windows
 #' scan <- scan.test(formula = formula, data = provinces_spain, case = "men", nv = 25,
 #' nsim = 99, distr = "bernoulli", windows ="elliptic")
@@ -146,7 +206,7 @@
 #' plot(scan, sf = FastFood.sf)
 #'
 #' # Case 4: DGP two categories
-#' N <- 1500
+#' N <- 150
 #' cx <- runif(N)
 #' cy <- runif(N)
 #' listw <- spdep::knearneigh(cbind(cx,cy), k = 10)
@@ -170,20 +230,45 @@
 #' distr = "multinomial", windows = "elliptic")
 #' print(scan)
 #' plot(scan)
+#'
+#' # Case 6: Flexible windows
+#' data(provinces_spain)
+#' sf::sf_use_s2(FALSE)
+#' provinces_spain$Male2Female <- factor(provinces_spain$Male2Female > 100)
+#' levels(provinces_spain$Male2Female) = c("men","woman")
+#' formula <- ~ Male2Female
+#' listw <- spdep::poly2nb(provinces_spain, queen = FALSE)
+#' scan <- scan.test(formula = formula, data = provinces_spain, case="men", listw = listw, nv = 6,
+#'                   nsim = 99, distr = "bernoulli", windows = "flexible")
+#' print(scan)
+#' summary(scan)
+#' plot(scan, sf = provinces_spain)
 #' }
 
 scan.test <- function(formula = NULL, data = NULL, fx = NULL, coor = NULL, case = NULL,
-                      nv = NULL, nsim = NULL, distr = NULL, windows = "circular",
+                      nv = NULL, nsim = NULL, distr = NULL, windows = "circular", listw = NULL,
                       alternative = "High", minsize = 1, control = list()) {
 
-  # profvis({
   if (is.null(distr))
     stop("Select a distribution, bernoulli or multinomial")
 
   coor.input <- coor
   distr <- match.arg(distr, c("bernoulli", "multinomial"))
-  windows <- match.arg(windows, c("circular", "elliptic"))
-
+  windows <- match.arg(windows, c("circular", "elliptic","flexible"))
+  # if flexible windows
+  if (windows=="flexible" && is.null(listw)) stop("A list of neigbourhood must be included")
+  ## Define the W matrix
+  if (windows=="flexible"){
+    if (inherits(listw, "knn")){
+      listw <- nb2listw(knn2nb(listw),zero.policy = TRUE)
+    } else if (inherits(listw, "matrix")){
+      listw <- (listw > 0)*1
+      listw <- mat2listw(listw)
+    } else if (inherits(listw, "nb")){
+      listw <- nb2listw(listw,zero.policy = TRUE)
+    }
+  }
+  #'
   # Select the arguments: (formula + data) or bien incluye la variable (fx)
   if (!is.null(formula) && !is.null(data)) {
     if (inherits(data, "Spatial")) data <- as(data, "sf")
@@ -244,7 +329,9 @@ scan.test <- function(formula = NULL, data = NULL, fx = NULL, coor = NULL, case
   if (windows=="elliptic"){
     nn <- suppressWarnings(nn_ellipse(coor = cbind(cx,cy), nv = nv, p = 30)$ellipses)
   }
-
+  if (windows=="flexible"){
+    nn <- nn_flexible(W = listw$neighbours, nv = nv)
+  }
   # XF <- matrix(mfx[nn], ncol = nv, nrow = N)
   XF <- matrix(mfx[nn], ncol = nv, nrow = dim(nn)[1])
 
@@ -287,8 +374,10 @@ scan.test <- function(formula = NULL, data = NULL, fx = NULL, coor = NULL, case
     #   MLC <- nn[a[1,1],1:a[1,2]]
     #   loglik <- max(lnlz)} ## With out restrictions in the number of cases in the MLC
     # else{
+
     a <- which(lnlz[,minsize:nv] == max(lnlz[,minsize:nv]), arr.ind = TRUE)
-    MLC <- nn[a[1,1],1:(minsize+a[1,2])]
+    # MLC <- nn[a[1,1],1:(minsize+a[1,2])]
+    MLC <- nn[a[1,1],1:min(dim(nn)[2],minsize+a[1,2])]
     loglik <- max(lnlz[,minsize:nv])
     # }
     a2 <- a
@@ -370,43 +459,16 @@ scan.test <- function(formula = NULL, data = NULL, fx = NULL, coor = NULL, case
     if (!is.null(seedinit)) set.seed(seedinit)
     scan.mc <- rep(0,nsim)
 
-    # cl <- parallel::makeCluster(detectCores())
-    # doParallel::registerDoParallel(cl)
-    # scan.mc <- foreach(1:nsim,.combine = rbind) %dopar% {
-    #   fxp <- mfx[sample(N)]
-    #   XF <- matrix(fxp[nn], ncol = nv, nrow = dim(nn)[1])
-    #   oz <- t(apply(XF==case, 1 , cumsum))
-    #   oznz <- oz/nz
-    #   OozNnz <- (O-oz)/(N-nz)
-    #   OozNnz.1 <- 1 - OozNnz
-    #   OozNnz.1[OozNnz.1 < 0] <- 0
-    #   a <- log(oznz)
-    #   a[a==-Inf] <- 0
-    #   b <- log(1-oznz)
-    #   b[b==-Inf] <- 0
-    #   c <- log(OozNnz)
-    #   c[c==-Inf] <- 0
-    #   d <- log(OozNnz.1)
-    #   d[d==-Inf] <- 0
-    #   if (alternative == "Both"){
-    #     lnlz <- exp(oz*a + (nz-oz)*b + (O-oz)*c + (N-O-nz+oz)*d)
-    #   }
-    #   if (alternative == "High"){
-    #     lnlz <- exp(oz*a + (nz-oz)*b + (O-oz)*c + (N-O-nz+oz)*d)*(oznz > OozNnz)
-    #   }
-    #   if (alternative == "Low"){
-    #     lnlz <- exp(oz*a + (nz-oz)*b + (O-oz)*c + (N-O-nz+oz)*d)*(oznz < OozNnz)
-    #   }
-    #   lnlz <- log(lnlz/lnlz0)
-    #   lnlz[lnlz==-Inf] <- 0
-    #   lnlz[is.na(lnlz)] <- 0
-    #   # scan.mc[f] <- max(lnlz)
-    #   scan.mc <- as.numeric(max(lnlz))
-    # }
     for (f in 1:nsim){
       fxp <- mfx[sample(N)]
+
       XF <- matrix(fxp[nn], ncol = nv, nrow = dim(nn)[1])
       oz <- t(apply(XF==case, 1 , cumsum))
+      # XF2 <- XF == case
+      # XF2 <- split(XF2, row(XF2))
+      # oz2 <- lapply(XF2, cumsum)
+      # oz2 <- do.call("rbind", oz2)
+      # oz <- t(apply_cumsum_col(t(XF == case)))
       oznz <- oz/nz
       OozNnz <- (O-oz)/(N-nz)
       OozNnz.1 <- 1 - OozNnz
@@ -486,8 +548,11 @@ scan.test <- function(formula = NULL, data = NULL, fx = NULL, coor = NULL, case
     loglik.second[f] <- max(LNLZ[,minsize:nv], na.rm = TRUE)
     cc <- which(LNLZ == max(LNLZ[,minsize:nv], na.rm = TRUE), arr.ind = TRUE)
     # Este es el cluster secundario
-    MLC2[[f]] <- nn[cc[1,1],1:(minsize+cc[1,2])]
-    mlc <- c(mlc,nn[cc[1,1],1:(minsize+cc[1,2])])
+
+    # MLC2[[f]] <- nn[cc[1,1],1:(minsize+cc[1,2])]
+    # mlc <- c(mlc,nn[cc[1,1],1:(minsize+cc[1,2])])
+    MLC2[[f]] <- nn[cc[1,1],1:min(dim(nn)[2],(minsize+cc[1,2]))]
+    mlc <- c(mlc,nn[cc[1,1],1:min(dim(nn)[2],(minsize+cc[1,2]))])
   }
   rm(pp,LNLZ,cc,mlc)
 
@@ -534,5 +599,4 @@ scan.test <- function(formula = NULL, data = NULL, fx = NULL, coor = NULL, case
 
   class(scan) <- c("htest","scantest")
   scan
- # })
 }
